==> autonomize_autorag-0.1.19.tar.gz: autonomize_autorag-0.1.19/autorag/language_models/openai/base_openai.py <==
"""OpenAI Embedding module implementation"""

# pylint: disable=line-too-long, duplicate-code

import os
from typing import Any, Dict, List, Optional, Union, override

try:
    from openai import AsyncOpenAI, OpenAI
    from openai._types import NOT_GIVEN
    from openai.types.chat.chat_completion import ChatCompletion
except ImportError as err:  # pragma: no cover
    raise ImportError(  # pragma: no cover
        "Unable to locate openai package. "
        'Please install it with `pip install "autonomize-autorag[openai]"`.'
    ) from err

from autorag.language_models.base import LanguageModel
from autorag.types.language_models import Generation, ToolCall, Usage


class OpenAILanguageModel(LanguageModel):
    """
    Implementation of the LanguageModel abstract base class using OpenAI's API.

    This class utilizes OpenAI's API to generate text data. It supports different models
    provided by OpenAI for text generation tasks like conversation and RAG systems.

    Example:
    .. code-block:: python

        from autorag.language_models.openai import OpenAILanguageModel

        model_name = "gpt-4o"
        llm = OpenAILanguageModel()

        llm.generate(
            messages=[{"role": "user", "content": "What is RAG?"}],
            model_name=model_name
        )
    """

    def __init__(self, api_key: Optional[str] = None):
        """
        Initializes the OpenAI language model client.

        Args:
            api_key (Optional[str]): OpenAI API key. If not provided, the key is read from the environment variable 'OPENAI_API_KEY'.

        Raises:
            ValueError: If the API key is not provided and cannot be found in the environment.
        """
        super().__init__()

        self.api_key = api_key or os.getenv("OPENAI_API_KEY")

        if not self.api_key:
            raise ValueError(
                "The api_key client option must be set either by passing api_key to the client or by setting the OPENAI_API_KEY environment variable."
            )

        self._client = OpenAI(
            api_key=self.api_key,
        )
        self._aclient = AsyncOpenAI(
            api_key=self.api_key,
        )

    def _generate(
        self,
        messages: List[Dict[str, Any]],
        model: str,
        tool: Optional[Dict[str, Any]] = None,
        max_tokens: Optional[int] = None,
        frequency_penalty: Optional[float] = None,
        presence_penalty: Optional[float] = None,
        seed: Optional[int] = None,
        stop: Optional[Union[str, List[str]]] = None,
        temperature: Optional[float] = None,
        **kwargs: Any,
    ) -> Generation:
        """
        Generates a response based on the provided messages and model settings.

        Args:
            messages (List[Dict[str, Any]]): A list of dictionaries containing the conversation history or prompts.
            model (str): The model identifier to use for text generation.
            tool (Optional[Dict[str, Any]]): A dictionary containing tool definition, if applicable.
            max_tokens (Optional[int]): Maximum number of tokens to generate. Defaults to no limit.
            frequency_penalty (Optional[float]): Penalizes new tokens based on their frequency in the generated text.
            presence_penalty (Optional[float]): Penalizes new tokens based on whether they appear in the generated text.
            seed (Optional[int]): Random seed for reproducibility of results.
            stop (Optional[Union[str, List[str]]]): String or list of strings to end the generation when encountered.
            temperature (Optional[float]): Controls randomness in the generation. Higher values (e.g., 1.0) make output more random, lower values (e.g., 0.2) make it more deterministic.
            **kwargs (Any): Additional optional arguments for further customization.

        Returns:
            Generation: Generated response based on the input messages and model parameters.
        """
        response: ChatCompletion = self._client.chat.completions.create(  # type: ignore[call-overload]
            messages=messages,  # type: ignore[arg-type]
            model=model,
            tools=[tool] if tool else NOT_GIVEN,  # type: ignore[list-item]
            max_tokens=max_tokens if max_tokens else NOT_GIVEN,
            frequency_penalty=frequency_penalty if frequency_penalty else NOT_GIVEN,
            presence_penalty=presence_penalty if presence_penalty else NOT_GIVEN,
            seed=seed if seed else NOT_GIVEN,
            stop=stop if stop else NOT_GIVEN,
            temperature=temperature if temperature else NOT_GIVEN,
            **kwargs,
        )

        output = Generation(
            content=response.choices[0].message.content,
            finish_reason=response.choices[0].finish_reason,
            role=response.choices[0].message.role,
            # To calculate usage for prompt and response
            usage=Usage(
                completion_tokens=(
                    response.usage.completion_tokens
                    if response.usage and response.usage.completion_tokens is not None
                    else None
                ),
                prompt_tokens=(
                    response.usage.prompt_tokens
                    if response.usage and response.usage.prompt_tokens is not None
                    else None
                ),
                total_tokens=(
                    response.usage.total_tokens
                    if response.usage and response.usage.total_tokens is not None
                    else None
                ),
            ),
            # If tool is used, then return its name and arguments, otherwise None
            tool_call=(
                ToolCall(
                    name=response.choices[0].message.tool_calls[0].function.name,
                    arguments=response.choices[0]
                    .message.tool_calls[0]
                    .function.arguments,
                )
                if response.choices[0].message.tool_calls
                and len(response.choices[0].message.tool_calls) > 0
                else None
            ),
        )
        return output

    @override
    async def agenerate(
        self,
        messages: List[Dict[str, Any]],
        model: str,
        tool: Optional[Dict[str, Any]] = None,
        max_tokens: Optional[int] = None,
        frequency_penalty: Optional[float] = None,
        presence_penalty: Optional[float] = None,
        seed: Optional[int] = None,
        stop: Optional[Union[str, List[str]]] = None,
        temperature: Optional[float] = None,
        **kwargs: Any,
    ) -> Generation:
        """
        Asynchronously generates a response based on the provided messages and model settings.

        Args:
            messages (List[Dict[str, Any]]): A list of dictionaries containing the conversation history or prompts.
            model (str): The model identifier to use for text generation.
            tool (Optional[Dict[str, Any]]): A dictionary containing tool definition, if applicable.
            max_tokens (Optional[int]): Maximum number of tokens to generate. Defaults to no limit.
            frequency_penalty (Optional[float]): Penalizes new tokens based on their frequency in the generated text.
            presence_penalty (Optional[float]): Penalizes new tokens based on whether they appear in the generated text.
            seed (Optional[int]): Random seed for reproducibility of results.
            stop (Optional[Union[str, List[str]]]): String or list of strings to end the generation when encountered.
            temperature (Optional[float]): Controls randomness in the generation. Higher values (e.g., 1.0) make output more random, lower values (e.g., 0.2) make it more deterministic.
            **kwargs (Any): Additional optional arguments for further customization.

        Returns:
            Generation: Generated response based on the input messages and model parameters.
        """
        response: ChatCompletion = await self._aclient.chat.completions.create(  # type: ignore[call-overload]
            messages=messages,  # type: ignore[arg-type]
            model=model,
            tools=[tool] if tool else NOT_GIVEN,  # type: ignore[list-item]
            max_tokens=max_tokens if max_tokens else NOT_GIVEN,
            frequency_penalty=frequency_penalty if frequency_penalty else NOT_GIVEN,
            presence_penalty=presence_penalty if presence_penalty else NOT_GIVEN,
            seed=seed if seed else NOT_GIVEN,
            stop=stop if stop else NOT_GIVEN,
            temperature=temperature if temperature else NOT_GIVEN,
            **kwargs,
        )

        output = Generation(
            content=response.choices[0].message.content,
            finish_reason=response.choices[0].finish_reason,
            role=response.choices[0].message.role,
            # To calculate usage for prompt and response
            usage=Usage(
                completion_tokens=(
                    response.usage.completion_tokens
                    if response.usage and response.usage.completion_tokens is not None
                    else None
                ),
                prompt_tokens=(
                    response.usage.prompt_tokens
                    if response.usage and response.usage.prompt_tokens is not None
                    else None
                ),
                total_tokens=(
                    response.usage.total_tokens
                    if response.usage and response.usage.total_tokens is not None
                    else None
                ),
            ),
            # If tool is used, then return its name and arguments, otherwise None
            tool_call=(
                ToolCall(
                    name=response.choices[0].message.tool_calls[0].function.name,
                    arguments=response.choices[0]
                    .message.tool_calls[0]
                    .function.arguments,
                )
                if response.choices[0].message.tool_calls
                and len(response.choices[0].message.tool_calls) > 0
                else None
            ),
        )
        return output

==> autonomize_autorag-0.1.34.tar.gz: autonomize_autorag-0.1.34/autorag/language_models/openai/base_openai.py <==
"""OpenAI Embedding module implementation"""

# pylint: disable=line-too-long, duplicate-code

import os
from typing import Any, Dict, List, Optional, Union, override

try:
    from openai import AsyncOpenAI, OpenAI
    from openai._types import NOT_GIVEN
    from openai.types.chat.chat_completion import ChatCompletion
except ImportError as err:  # pragma: no cover
    raise ImportError(  # pragma: no cover
        "Unable to locate openai package. "
        'Please install it with `pip install "autonomize-autorag[openai]"`.'
    ) from err

from autorag.language_models.base import LanguageModel
from autorag.types.language_models import Generation, ToolCall, Usage


class OpenAILanguageModel(LanguageModel):
    """
    Implementation of the LanguageModel abstract base class using OpenAI's API.

    This class utilizes OpenAI's API to generate text data. It supports different models
    provided by OpenAI for text generation tasks like conversation and RAG systems.

    Example:
    .. code-block:: python

        from autorag.language_models import OpenAILanguageModel

        model_name = "gpt-4o"
        llm = OpenAILanguageModel()

        llm.generate(
            messages=[{"role": "user", "content": "What is RAG?"}],
            model_name=model_name
        )
    """

    def __init__(self, api_key: Optional[str] = None):
        """
        Initializes the OpenAI language model client.

        Args:
            api_key (Optional[str]): OpenAI API key. If not provided, the key is read from the environment variable 'OPENAI_API_KEY'.

        Raises:
            ValueError: If the API key is not provided and cannot be found in the environment.
        """
        super().__init__()

        self.api_key = api_key or os.getenv("OPENAI_API_KEY")

        if not self.api_key:
            raise ValueError(
                "The api_key client option must be set either by passing api_key to the client or by setting the OPENAI_API_KEY environment variable."
            )

        self._client = OpenAI(
            api_key=self.api_key,
        )
        self._aclient = AsyncOpenAI(
            api_key=self.api_key,
        )

    def _generate(
        self,
        messages: List[Dict[str, Any]],
        model: str,
        tool: Optional[Dict[str, Any]] = None,
        max_tokens: Optional[int] = None,
        frequency_penalty: Optional[float] = None,
        presence_penalty: Optional[float] = None,
        seed: Optional[int] = None,
        stop: Optional[Union[str, List[str]]] = None,
        temperature: Optional[float] = None,
        **kwargs: Any,
    ) -> Generation:
        """
        Generates a response based on the provided messages and model settings.

        Args:
            messages (List[Dict[str, Any]]): A list of dictionaries containing the conversation history or prompts.
            model (str): The model identifier to use for text generation.
            tool (Optional[Dict[str, Any]]): A dictionary containing tool definition, if applicable.
            max_tokens (Optional[int]): Maximum number of tokens to generate. Defaults to no limit.
            frequency_penalty (Optional[float]): Penalizes new tokens based on their frequency in the generated text.
            presence_penalty (Optional[float]): Penalizes new tokens based on whether they appear in the generated text.
            seed (Optional[int]): Random seed for reproducibility of results.
            stop (Optional[Union[str, List[str]]]): String or list of strings to end the generation when encountered.
            temperature (Optional[float]): Controls randomness in the generation. Higher values (e.g., 1.0) make output more random, lower values (e.g., 0.2) make it more deterministic.
            **kwargs (Any): Additional optional arguments for further customization.

        Returns:
            Generation: Generated response based on the input messages and model parameters.
        """
        response: ChatCompletion = self._client.chat.completions.create(  # type: ignore[call-overload]
            messages=messages,  # type: ignore[arg-type]
            model=model,
            tools=[tool] if tool else NOT_GIVEN,  # type: ignore[list-item]
            max_tokens=max_tokens if max_tokens else NOT_GIVEN,
            frequency_penalty=frequency_penalty if frequency_penalty else NOT_GIVEN,
            presence_penalty=presence_penalty if presence_penalty else NOT_GIVEN,
            seed=seed if seed else NOT_GIVEN,
            stop=stop if stop else NOT_GIVEN,
            temperature=temperature if temperature else NOT_GIVEN,
            **kwargs,
        )

        output = Generation(
            content=response.choices[0].message.content,
            finish_reason=response.choices[0].finish_reason,
            role=response.choices[0].message.role,
            # To calculate usage for prompt and response
            usage=Usage(
                completion_tokens=(
                    response.usage.completion_tokens
                    if response.usage and response.usage.completion_tokens is not None
                    else None
                ),
                prompt_tokens=(
                    response.usage.prompt_tokens
                    if response.usage and response.usage.prompt_tokens is not None
                    else None
                ),
                total_tokens=(
                    response.usage.total_tokens
                    if response.usage and response.usage.total_tokens is not None
                    else None
                ),
            ),
            # If tool is used, then return its name and arguments, otherwise None
            tool_call=(
                ToolCall(
                    name=response.choices[0].message.tool_calls[0].function.name,
                    arguments=response.choices[0]
                    .message.tool_calls[0]
                    .function.arguments,
                )
                if response.choices[0].message.tool_calls
                and len(response.choices[0].message.tool_calls) > 0
                else None
            ),
        )
        return output

    @override
    async def agenerate(
        self,
        messages: List[Dict[str, Any]],
        model: str,
        tool: Optional[Dict[str, Any]] = None,
        max_tokens: Optional[int] = None,
        frequency_penalty: Optional[float] = None,
        presence_penalty: Optional[float] = None,
        seed: Optional[int] = None,
        stop: Optional[Union[str, List[str]]] = None,
        temperature: Optional[float] = None,
        **kwargs: Any,
    ) -> Generation:
        """
        Asynchronously generates a response based on the provided messages and model settings.

        Args:
            messages (List[Dict[str, Any]]): A list of dictionaries containing the conversation history or prompts.
            model (str): The model identifier to use for text generation.
            tool (Optional[Dict[str, Any]]): A dictionary containing tool definition, if applicable.
            max_tokens (Optional[int]): Maximum number of tokens to generate. Defaults to no limit.
            frequency_penalty (Optional[float]): Penalizes new tokens based on their frequency in the generated text.
            presence_penalty (Optional[float]): Penalizes new tokens based on whether they appear in the generated text.
            seed (Optional[int]): Random seed for reproducibility of results.
            stop (Optional[Union[str, List[str]]]): String or list of strings to end the generation when encountered.
            temperature (Optional[float]): Controls randomness in the generation. Higher values (e.g., 1.0) make output more random, lower values (e.g., 0.2) make it more deterministic.
            **kwargs (Any): Additional optional arguments for further customization.

        Returns:
            Generation: Generated response based on the input messages and model parameters.
        """
        response: ChatCompletion = await self._aclient.chat.completions.create(  # type: ignore[call-overload]
            messages=messages,  # type: ignore[arg-type]
            model=model,
            tools=[tool] if tool else NOT_GIVEN,  # type: ignore[list-item]
            max_tokens=max_tokens if max_tokens else NOT_GIVEN,
            frequency_penalty=frequency_penalty if frequency_penalty else NOT_GIVEN,
            presence_penalty=presence_penalty if presence_penalty else NOT_GIVEN,
            seed=seed if seed else NOT_GIVEN,
            stop=stop if stop else NOT_GIVEN,
            temperature=temperature if temperature else NOT_GIVEN,
            **kwargs,
        )

        output = Generation(
            content=response.choices[0].message.content,
            finish_reason=response.choices[0].finish_reason,
            role=response.choices[0].message.role,
            # To calculate usage for prompt and response
            usage=Usage(
                completion_tokens=(
                    response.usage.completion_tokens
                    if response.usage and response.usage.completion_tokens is not None
                    else None
                ),
                prompt_tokens=(
                    response.usage.prompt_tokens
                    if response.usage and response.usage.prompt_tokens is not None
                    else None
                ),
                total_tokens=(
                    response.usage.total_tokens
                    if response.usage and response.usage.total_tokens is not None
                    else None
                ),
            ),
            # If tool is used, then return its name and arguments, otherwise None
            tool_call=(
                ToolCall(
                    name=response.choices[0].message.tool_calls[0].function.name,
                    arguments=response.choices[0]
                    .message.tool_calls[0]
                    .function.arguments,
                )
                if response.choices[0].message.tool_calls
                and len(response.choices[0].message.tool_calls) > 0
                else None
            ),
        )
        return output
